/*
 * lab5.c
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


/*****************************************************************************
 * Preprocessor definitions
 ****************************************************************************/

/* Tensiunea de referinta utilizata de ADC. */
#define ADC_AREF_VOLTAGE 3.3
#define ADC_MAX_CHANNELS 6

/* Decomentati linia pentru a activa ADC IRQ. */
//#define ADC_USE_IRQ

/*****************************************************************************
 * Private types
 ****************************************************************************/

/* Numele canalelor de ADC. */
typedef struct
{
  const char *name;
  uint8_t    channel;
} adc_channel_t;

/*****************************************************************************
 * Private data
 ****************************************************************************/

/* Perechi de (Nume canal, Numar canal). */
const adc_channel_t ADC_channels[] = {

  // Senzor de lumina conectat la ADC1
  {"Lumina", 0},
};

/* Numarul de canale ADC utilizate. */
const int adc_num_channels = sizeof(ADC_channels) / sizeof(ADC_channels[0]);

#ifdef ADC_USE_IRQ
/* Numarul canalului initial. */
#  define ADC_INIT_CHANNEL 0

/* Valorile citite de la fiecare canal. */
volatile uint16_t ADC_value[ADC_MAX_CHANNELS];
#endif


/*
 * Functia initializeaza convertorul Analog-Digital.
 */
void ADC_init(void)
{
#ifndef ADC_USE_IRQ
    // TODO 2
    // enable ADC with:
    // * reference AVCC with external capacitor at AREF pin
    // * without left adjust of conversion result
    // * no auto-trigger
    // * no interrupt
    // * prescaler at 32
    ADMUX = (1 << REFS0);
    ADCSRA = (1 << ADEN) | (5 << ADPS0);
#else
    // TODO 4
    // enable ADC with interrupt, the rest of the settings like in TODO 2
    ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
    ADCSRA = (1 << ADEN) | (1 << ADIE) | (5 << ADPS0);
#endif
}

/*
 * Functia porneste o noua conversie pentru canalul precizat.
 * In modul fara intreruperi, apelul functiei este blocant. Aceasta se
 * intoarce cand conversia este finalizata.
 *
 * In modul cu intreruperi apelul NU este blocant.
 *
 * @return Valoarea numerica raw citita de ADC de pe canlul specificat.
 */
uint16_t ADC_get(uint8_t channel)
{
#ifndef ADC_USE_IRQ
    // TODO 2
    // start ADC conversion on "channel"
    // wait for completion
    // return the result
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
#else
    //TODO 4
    return ADC_value[channel];
#endif

    (void)channel;
    return 0;
}

/*
 * Functia primeste o valoare numerica raw citita de convertul Analog-Digital
 * si calculeaza tensiunea (in volti) pe baza tensiunei de referinta.
 */
double ADC_voltage(int raw)
{
    (void)raw;
    // TODO 2
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}

/*
 * Functia afiseaza pe display-ul grafic numele canalelor ADC si valorile
 * citite de la acestea.
 */
/*void ADC_show(void)
{
    char buf[32];

    // La ADC-ul microcontroller-ului avem conectate urmatoarele:
    //   ADC0: senzor temperatura
    //   ADC1: senzor lumina
    //   ADC5: 6 butoane + rezeistente (divizor de tensiune)

    // TODO 1: Afisati pe display-ul grafic numele fiecarui canalelor, cate
    // unul pe fiecare linie. Folositi-va de array-ul 'ADC_channels'
    //
    // Inaltimea unei linii este de ST7735R_FONT_HEIGHT pixeli (15 pixeli)
    // Afisati incepand de la coordonatele (0, 0)

    uint8_t y = 5;
    for (int i = 0; i < adc_num_channels; i++)
    {
        const adc_channel_t *channel = &ADC_channels[i];

        sprintf(buf, "%s: %.2fV", channel->name,
                ADC_voltage(ADC_get(channel->channel)));

        // Print text, culoare alb pe background negru
        ST7735R_DrawText(0, y, buf, 255, 255, 255, 0, 0, 0);

        // Mergem la linia urmatoare
        y += ST7735R_FONT_HEIGHT;
    }
}*/

